package ch.sahits.game.tetris.model;

import java.awt.Color;

import ch.sahits.game.tetris.ui.TetrisConstants;
/**
 * The playing field is a matrix indicating which coordinate is occupied by which
 * color. If a field is null, it is unoccupied.
 * @author andi
 *
 */
public class PlayingField {

	private Color[][] matrix = new Color[TetrisConstants.NUMBER_OF_ROWS][TetrisConstants.BLOCKS_PER_ROW];
	/**
	 * Retrieve row number i.
	 * @param i
	 * @return
	 */
	public Color[] getRow(int i){
		return matrix[i];
	}
	/**
	 * Get number of rows.
	 * @return
	 */
	public int getNbRows(){
		return TetrisConstants.NUMBER_OF_ROWS;
	}
	/**
	 * Get number of columns.
	 * @return
	 */
	public int getNbColumns(){
		return TetrisConstants.BLOCKS_PER_ROW;
	}
	/**
	 * Set a color at the coordinate.
	 * @param x
	 * @param y
	 * @param c
	 */
	public void set(int x, int y, Color c){
		matrix[y][x] = c;
	}
	/**
	 * Retrieve the color of coordinate.
	 * @param x
	 * @param y
	 * @return
	 */
	public Color get(int x, int y){
		return matrix[y][x];
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (int y=0;y<TetrisConstants.NUMBER_OF_ROWS;y++){
			sb.append("[");
			for (int x = 0;x<TetrisConstants.BLOCKS_PER_ROW;x++){
				Color color = get(x, y);
				sb.append(color==null?"-":"X");
				if (x<TetrisConstants.BLOCKS_PER_ROW-1){
					sb.append(",");
				}
			}
			sb.append("] "+y+"\n");
		}
		return sb.toString();
	}
	
	
}
