package ch.sahits.game.tetris.ui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Iterator;

import javax.swing.JFrame;

import ch.sahits.game.event.Event;
import ch.sahits.game.rendering.BaseView;
import ch.sahits.game.rendering.RenderablePart;
/**
 * The main view. The game view delegates its work to the {@link PictureView} 
 * in the left and the {@link TetrisView} in the right.
 * @author andi
 *
 */
public class GameView extends BaseView {
	


	public GameView(Rectangle bounds, JFrame parent) {
		super(bounds, parent);
		createPictureView();
		createGameView();
	}
	/**
	 * Create the tetris view
	 */
	private void createGameView() {
		TetrisView tetris = new TetrisView(computeTetrisBounds());
		addPart(tetris);
	}
	/**
	 * Compute the bounds for the tetris view
	 * @return
	 */
	private Rectangle computeTetrisBounds(){
		Rectangle rect = getBounds();
		int x = rect.x+rect.width-2*TetrisConstants.BORDER_THICKNESS-TetrisConstants.BLOCK_SIZE*TetrisConstants.BLOCKS_PER_ROW-TetrisConstants.MARGIN*2;
		int y = rect.y;
		return new Rectangle(x, y, rect.width-x, rect.height);
	}
	/**
	 * Create the picture view
	 */
	private void createPictureView() {
		Rectangle rect = getBounds();
		Rectangle tetris = computeTetrisBounds();
		int width = rect.width  - tetris.width;
		PictureView picture;
		try {
			picture = new PictureView(new Rectangle(rect.x, rect.y+30, width, rect.height-30));
			addPart(picture);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
		
	}
	@Override
	public void gameRender(Graphics gScr) {
		Color oldColor = gScr.getColor();
		for (Iterator<RenderablePart> iter = iterator(); iter.hasNext();) {
			RenderablePart part = iter.next();
			part.gameRender(gScr);
		}
		
		gScr.setColor(oldColor);
	}

	@Override
	public void gameUpdate(Event e, Object notice) {
		// ignore
	}

	@Override
	protected RenderablePart getActiveMainView() {
		// not used
		return null;
	}

}
