package ch.sahits.game.tetris.ui;

import static ch.sahits.game.tetris.ctrl.GameStateUtility.isGamePlayState;
import static ch.sahits.game.tetris.ctrl.GameStateUtility.isMenuState;

import java.awt.Color;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;

import ch.sahits.game.event.Event;
import ch.sahits.game.event.IEventListener;
import ch.sahits.game.rendering.MainFrame;
import ch.sahits.game.tetris.ctrl.GameStateUtility;
import ch.sahits.game.tetris.ctrl.TetrisController;
import ch.sahits.game.tetris.event.EGameState;
import ch.sahits.game.tetris.event.EKeyStroke;
import ch.sahits.game.tetris.event.GameStateChangeEvent;
import ch.sahits.game.tetris.event.KeyStrokeEvent;
import ch.sahits.game.tetris.event.PlayFieldUpdateEvent;
import ch.sahits.game.tetris.model.KeyMap;
import ch.sahits.game.tetris.model.TetrisViewModel;
/**
 * Main application providing the user interfase.
 * @author andi
 *
 */
public class TetrisMainFrame extends MainFrame implements IEventListener{

	private static final long serialVersionUID = -1946899277967686106L;
	private GameView gameview;
	private TetrisController controller;
	private EGameState currentState=EGameState.MENU; // Though it starts with START and then changes to menu, but these update are not yet registered
	
	private int upsCounter = 0;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		TetrisMainFrame frame = new TetrisMainFrame();
		frame.setVisible(true);
	}

	/**
	 * Create the frame.
	 */
	public TetrisMainFrame() {
		super("Tetris");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setExtendedState(JFrame.NORMAL);
		setBounds(50, 50, getRactangle().width-100, getRactangle().height-100);
		Rectangle drawable = computeRawableArea();
		setBackground(Color.BLACK);
		gameview = new GameView(drawable, this);
		addView(gameview);
		controller = new TetrisController();
		gameStart();
		Event.add(this);
	}

	/**
	 * {@inheritDoc}
	 * Decouple the frame rate from the update rate and
	 * call the update method of the controller
	 */
	@Override
	protected void gameUpdate() {
		if (upsCounter++%TetrisConstants.FPS_UPS_RATIO == 0){
			controller.gameUpdate();
		}
	}
	
	@Override
	protected void testKey(KeyEvent event) {
		if (GameStateUtility.isKeySetState(currentState)){
			new KeyStrokeEvent().notify(event);
		}
		if (isGamePlayState(currentState)){
			if (event.getKeyCode() == KeyMap.PAUSE){
				new KeyStrokeEvent().notify(EKeyStroke.PAUSE);
			}
			if (event.getKeyCode() == KeyMap.DOWN){
				new KeyStrokeEvent().notify(EKeyStroke.DOWN);
			}
			if (event.getKeyCode() == KeyMap.LEFT){
				new KeyStrokeEvent().notify(EKeyStroke.LEFT);
			}
			if (event.getKeyCode() == KeyMap.RIGHT){
				new KeyStrokeEvent().notify(EKeyStroke.RIGHT);
			}
			if (event.getKeyCode() == KeyMap.ROTATE) {
				new KeyStrokeEvent().notify(EKeyStroke.ROTATE);
			}
			if (event.getKeyCode() == KeyMap.ENTER){
				new KeyStrokeEvent().notify(EKeyStroke.ENTER);
			}
		}
 		if (isMenuState(currentState)){
			if (event.getKeyCode() == KeyMap.ONE){
				new GameStateChangeEvent(EGameState.SET_KEY_LEFT).notify(null);
			}
			if (event.getKeyCode() == KeyMap.TWO){
				new GameStateChangeEvent(EGameState.NEXT_LEVEL_INFO).notify(null);
			}
			if (event.getKeyCode() == KeyMap.THREE){
				new GameStateChangeEvent(EGameState.EXIT).notify(null);
				closeWindow();
			}
		}	
	}
	/**
	 * No mouse support
	 */
	@Override
	protected void testMove(int arg0, int arg1) {
		// no mouse support
	}
	/**
	 * No mouse support
	 */
	@Override
	protected void testPress(int arg0, int arg1) {
		// no mouse support
	}
	private Rectangle computeRawableArea() {
		Insets frameInsets = getInsets();
		Rectangle bounds = getBounds();
		Rectangle rect = new Rectangle(bounds.x+frameInsets.left, bounds.y+frameInsets.top, bounds.width-frameInsets.left-frameInsets.right, bounds.height-frameInsets.top-frameInsets.bottom);
		return rect;
	}

	/**
	 * {@inheritDoc}
	 * Initialize the controller and set up the proper game states.
	 */
	@Override
	protected void gameStart() {
		super.gameStart();
		new GameStateChangeEvent(EGameState.START).notify(null);
		TetrisViewModel model = controller.initializeViewModel();
		new PlayFieldUpdateEvent(model).notify(null);
		new GameStateChangeEvent(EGameState.MENU).notify(null);
	}
	/**
	 * Keep track of the game state changes
	 */
	public void gameUpdate(Event e, Object notice) {
		if (e instanceof GameStateChangeEvent){
			GameStateChangeEvent change = (GameStateChangeEvent) e;
			currentState = change.getNextGameState();
		}
	}


}
